@@ -28,8 +28,9 @@
 #include <sys/types.h>
 
 #include "bson-macros.h"
+#include "bson-config.h"
 #include "bson-compat.h"
-
+#include "bson-endian.h"
 
 BSON_BEGIN_DECLS
 
@@ -158,6 +159,30 @@ typedef struct
 } bson_oid_t;
 
 
+/**
+ * bson_decimal128_t:
+ *
+ * @high The high-order bytes of the decimal128.  This field contains sign,
+ *       combination bits, exponent, and part of the coefficient continuation.
+ * @low  The low-order bytes of the decimal128.  This field contains the second
+ *       part of the coefficient continuation.
+ *
+ * This structure is a boxed type containing the value for the BSON decimal128
+ * type.  The structure stores the 128 bits such that they correspond to the
+ * native format for the IEEE decimal128 type, if it is implemented.
+ **/
+typedef struct
+{
+#if BSON_BYTE_ORDER == BSON_LITTLE_ENDIAN
+   uint64_t low;
+   uint64_t high;
+#elif BSON_BYTE_ORDER == BSON_BIG_ENDIAN
+   uint64_t high;
+   uint64_t low;
+#endif
+} bson_decimal128_t;
+
+
 BSON_STATIC_ASSERT (sizeof (bson_oid_t) == 12);
 
 
@@ -210,6 +235,7 @@ typedef enum
    BSON_TYPE_INT32 = 0x10,
    BSON_TYPE_TIMESTAMP = 0x11,
    BSON_TYPE_INT64 = 0x12,
+   BSON_TYPE_DECIMAL128 = 0x13,
    BSON_TYPE_MAXKEY = 0x7F,
    BSON_TYPE_MINKEY = 0xFF,
 } bson_type_t;
@@ -300,6 +326,7 @@ typedef struct _bson_value_t
          char           *symbol;
          uint32_t        len;
       } v_symbol;
+      bson_decimal128_t  v_decimal128;
    } value;
 } bson_value_t
 BSON_ALIGNED_END (8);
@@ -344,11 +371,13 @@ BSON_ALIGNED_END (128);
  * incoming mongo packet.
  */
 
+BSON_ALIGNED_BEGIN (BSON_ALIGN_OF_PTR)
 typedef struct
 {
    uint32_t type;
    /*< private >*/
-} bson_reader_t;
+} bson_reader_t
+BSON_ALIGNED_END (BSON_ALIGN_OF_PTR);
 
 
 /**
@@ -369,113 +398,128 @@ typedef struct
 BSON_ALIGNED_BEGIN (8)
 typedef struct
 {
-   bool (*visit_before)     (const bson_iter_t *iter,
-                             const char        *key,
-                             void              *data);
-   bool (*visit_after)      (const bson_iter_t *iter,
-                             const char        *key,
-                             void              *data);
-   void (*visit_corrupt)    (const bson_iter_t *iter,
-                             void              *data);
-   bool (*visit_double)     (const bson_iter_t *iter,
-                             const char        *key,
-                             double             v_double,
-                             void              *data);
-   bool (*visit_utf8)       (const bson_iter_t *iter,
-                             const char        *key,
-                             size_t             v_utf8_len,
-                             const char        *v_utf8,
-                             void              *data);
-   bool (*visit_document)   (const bson_iter_t *iter,
-                             const char        *key,
-                             const bson_t      *v_document,
-                             void              *data);
-   bool (*visit_array)      (const bson_iter_t *iter,
-                             const char        *key,
-                             const bson_t      *v_array,
-                             void              *data);
-   bool (*visit_binary)     (const bson_iter_t *iter,
-                             const char        *key,
-                             bson_subtype_t     v_subtype,
-                             size_t             v_binary_len,
-                             const uint8_t     *v_binary,
-                             void              *data);
-   bool (*visit_undefined)  (const bson_iter_t *iter,
-                             const char        *key,
-                             void              *data);
-   bool (*visit_oid)        (const bson_iter_t *iter,
-                             const char        *key,
-                             const bson_oid_t  *v_oid,
-                             void              *data);
-   bool (*visit_bool)       (const bson_iter_t *iter,
-                             const char        *key,
-                             bool               v_bool,
-                             void              *data);
-   bool (*visit_date_time)  (const bson_iter_t *iter,
-                             const char        *key,
-                             int64_t            msec_since_epoch,
-                             void              *data);
-   bool (*visit_null)       (const bson_iter_t *iter,
-                             const char        *key,
-                             void              *data);
-   bool (*visit_regex)      (const bson_iter_t *iter,
-                             const char        *key,
-                             const char        *v_regex,
-                             const char        *v_options,
-                             void              *data);
-   bool (*visit_dbpointer)  (const bson_iter_t *iter,
-                             const char        *key,
-                             size_t             v_collection_len,
-                             const char        *v_collection,
-                             const bson_oid_t  *v_oid,
-                             void              *data);
-   bool (*visit_code)       (const bson_iter_t *iter,
-                             const char        *key,
-                             size_t             v_code_len,
-                             const char        *v_code,
-                             void              *data);
-   bool (*visit_symbol)     (const bson_iter_t *iter,
-                             const char        *key,
-                             size_t             v_symbol_len,
-                             const char        *v_symbol,
-                             void              *data);
-   bool (*visit_codewscope) (const bson_iter_t *iter,
-                             const char        *key,
-                             size_t             v_code_len,
-                             const char        *v_code,
-                             const bson_t      *v_scope,
-                             void              *data);
-   bool (*visit_int32)      (const bson_iter_t *iter,
-                             const char        *key,
-                             int32_t            v_int32,
-                             void              *data);
-   bool (*visit_timestamp)  (const bson_iter_t *iter,
-                             const char        *key,
-                             uint32_t           v_timestamp,
-                             uint32_t           v_increment,
-                             void              *data);
-   bool (*visit_int64)      (const bson_iter_t *iter,
-                             const char        *key,
-                             int64_t            v_int64,
-                             void              *data);
-   bool (*visit_maxkey)     (const bson_iter_t *iter,
-                             const char        *key,
-                             void              *data);
-   bool (*visit_minkey)     (const bson_iter_t *iter,
-                             const char        *key,
-                             void              *data);
+   /* run before / after descending into a document */
+   bool (*visit_before)           (const bson_iter_t *iter,
+                                   const char        *key,
+                                   void              *data);
+   bool (*visit_after)            (const bson_iter_t *iter,
+                                   const char        *key,
+                                   void              *data);
+   /* corrupt BSON, or unsupported type and visit_unsupported_type not set */
+   void (*visit_corrupt)          (const bson_iter_t *iter,
+                                   void              *data);
+   /* normal bson field callbacks */
+   bool (*visit_double)           (const bson_iter_t *iter,
+                                   const char        *key,
+                                   double             v_double,
+                                   void              *data);
+   bool (*visit_utf8)             (const bson_iter_t *iter,
+                                   const char        *key,
+                                   size_t             v_utf8_len,
+                                   const char        *v_utf8,
+                                   void              *data);
+   bool (*visit_document)         (const bson_iter_t *iter,
+                                   const char        *key,
+                                   const bson_t      *v_document,
+                                   void              *data);
+   bool (*visit_array)            (const bson_iter_t *iter,
+                                   const char        *key,
+                                   const bson_t      *v_array,
+                                   void              *data);
+   bool (*visit_binary)           (const bson_iter_t *iter,
+                                   const char        *key,
+                                   bson_subtype_t     v_subtype,
+                                   size_t             v_binary_len,
+                                   const uint8_t     *v_binary,
+                                   void              *data);
+   /* normal field with deprecated "Undefined" BSON type */
+   bool (*visit_undefined)        (const bson_iter_t *iter,
+                                   const char        *key,
+                                   void              *data);
+   bool (*visit_oid)              (const bson_iter_t *iter,
+                                   const char        *key,
+                                   const bson_oid_t  *v_oid,
+                                   void              *data);
+   bool (*visit_bool)             (const bson_iter_t *iter,
+                                   const char        *key,
+                                   bool               v_bool,
+                                   void              *data);
+   bool (*visit_date_time)        (const bson_iter_t *iter,
+                                   const char        *key,
+                                   int64_t            msec_since_epoch,
+                                   void              *data);
+   bool (*visit_null)             (const bson_iter_t *iter,
+                                   const char        *key,
+                                   void              *data);
+   bool (*visit_regex)            (const bson_iter_t *iter,
+                                   const char        *key,
+                                   const char        *v_regex,
+                                   const char        *v_options,
+                                   void              *data);
+   bool (*visit_dbpointer)        (const bson_iter_t *iter,
+                                   const char        *key,
+                                   size_t             v_collection_len,
+                                   const char        *v_collection,
+                                   const bson_oid_t  *v_oid,
+                                   void              *data);
+   bool (*visit_code)             (const bson_iter_t *iter,
+                                   const char        *key,
+                                   size_t             v_code_len,
+                                   const char        *v_code,
+                                   void              *data);
+   bool (*visit_symbol)           (const bson_iter_t *iter,
+                                   const char        *key,
+                                   size_t             v_symbol_len,
+                                   const char        *v_symbol,
+                                   void              *data);
+   bool (*visit_codewscope)       (const bson_iter_t *iter,
+                                   const char        *key,
+                                   size_t             v_code_len,
+                                   const char        *v_code,
+                                   const bson_t      *v_scope,
+                                   void              *data);
+   bool (*visit_int32)            (const bson_iter_t *iter,
+                                   const char        *key,
+                                   int32_t            v_int32,
+                                   void              *data);
+   bool (*visit_timestamp)        (const bson_iter_t *iter,
+                                   const char        *key,
+                                   uint32_t           v_timestamp,
+                                   uint32_t           v_increment,
+                                   void              *data);
+   bool (*visit_int64)            (const bson_iter_t *iter,
+                                   const char        *key,
+                                   int64_t            v_int64,
+                                   void              *data);
+   bool (*visit_maxkey)           (const bson_iter_t *iter,
+                                   const char        *key,
+                                   void              *data);
+   bool (*visit_minkey)           (const bson_iter_t *iter,
+                                   const char        *key,
+                                   void              *data);
+   /* if set, called instead of visit_corrupt when an apparently valid BSON
+    * includes an unrecognized field type (reading future version of BSON) */
+   void (*visit_unsupported_type) (const bson_iter_t *iter,
+                                   const char        *key,
+                                   uint32_t           type_code,
+                                   void              *data);
+   bool (*visit_decimal128)       (const bson_iter_t       *iter,
+                                   const char              *key,
+                                   const bson_decimal128_t *v_decimal128,
+                                   void                    *data);
 
-   void *padding[9];
+   void *padding[7];
 } bson_visitor_t
 BSON_ALIGNED_END (8);
 
+#define BSON_ERROR_BUFFER_SIZE 504
 
 BSON_ALIGNED_BEGIN (8)
 typedef struct _bson_error_t
 {
    uint32_t domain;
    uint32_t code;
-   char     message[504];
+   char     message[BSON_ERROR_BUFFER_SIZE];
 } bson_error_t
 BSON_ALIGNED_END (8);
 
