@@ -19,7 +19,7 @@ use warnings;
 package MongoDB::_URI;
 
 use version;
-our $VERSION = 'v1.6.0';
+our $VERSION = 'v1.6.1';
 
 use Moo;
 use MongoDB::Error;
@@ -97,6 +97,7 @@ sub _build_valid_options {
             appName
             authMechanism
             authMechanismProperties
+            authSource
             connectTimeoutMS
             connect
             heartbeatFrequencyMS
@@ -221,6 +222,9 @@ sub BUILD {
             if ( $lc_k eq 'authmechanismproperties' ) {
                 $parsed{$lc_k} = _parse_doc( $k, $v );
             }
+            elsif ( $lc_k eq 'authsource' ) {
+                $result{db_name} = $v;
+            }
             elsif ( $lc_k eq 'readpreferencetags' ) {
                 $parsed{$lc_k} ||= [];
                 push @{ $parsed{$lc_k} }, _parse_doc( $k, $v );
